//Clock
#include "device_io.h"
#include <stdint.h>
#include <stdio.h>

const int CLK = 4;
const int IO = 5;
const int CE = 6;

typedef enum
{
	SEC,
	MIN,
	HR,
	DAY,
	MONTH,
	DOW,
	YEAR,
	CONTROL,
	TRICKLE,
	BURST
} commands;

typedef enum
{
	REG,
	RAM
} modes;

typedef struct
{
	uint8_t month;
	uint8_t day;
	uint8_t year;
	uint8_t dow;
	uint8_t hour;
	uint8_t minute;
	uint8_t second;
} datetime;

uint8_t command(int clk_ram, int reg);
void write(uint8_t cmd, uint8_t data);
uint8_t read(uint8_t cmd);
void setDateTime(datetime dt);
datetime getDateTime(void);
uint8_t bin2bcd(uint8_t value);
uint8_t bcd2bin(uint8_t value);

void main (int argc,  char* argv[])
{ 
   int val;
	datetime dt = {12, 31, 11, 7, 23, 59, 30};
	uint8_t csec;


   Out(CE, LOW);
	Out(CLK, LOW);

   write(command(REG, CONTROL), 0);
   write(command(REG, SEC), read(command(REG, SEC)) & 0x7F);
   write(command(REG, TRICKLE), 0);

	setDateTime(dt);
	csec = dt.second;

	for(;;)
	{
		dt = getDateTime();
		if (csec != dt.second)
		{
			csec = dt.second;
			printf("%d %d %d %d  %d %d %d\n", dt.dow, dt.month, dt.day, dt.year, dt.hour, dt.minute, dt.second);
			
		}
	}
	
}

uint8_t command(int clk_ram, int reg)
{
	uint8_t val = 0x80 + (clk_ram << 6) + (reg << 1);
	return val;
}

void write(uint8_t cmd, uint8_t data)
{
	cmd &= 0x8E;
	Out(CE, HIGH);
	ShiftOut(IO, CLK, (uint32_t)cmd, LSBFIRST, 8);
   ShiftOut(IO, CLK, (uint32_t)data, LSBFIRST, 8);
	Out(CE, LOW);
}

uint8_t read(uint8_t cmd)
{
	uint8_t val = 0;
	cmd |= 1;
	Out(CE, HIGH);
	ShiftOut(IO, CLK, (uint32_t)cmd, LSBFIRST, 8);
	val = ShiftIn(IO, CLK, LSBPRE, 8);
	Out(CE, LOW);
	return val;
}

void setDateTime(datetime dt)
{
	write(command(REG, MONTH), bin2bcd(dt.month));
	write(command(REG, DAY), bin2bcd(dt.day));
	write(command(REG, YEAR), bin2bcd(dt.year));
	write(command(REG, DOW), dt.dow);
	write(command(REG, HR), bin2bcd(dt.hour));
	write(command(REG, MIN), bin2bcd(dt.minute));
	write(command(REG, SEC), bin2bcd(dt.second));
}

datetime getDateTime(void)
{
	datetime dt;
	uint8_t temp;

	dt.month = bcd2bin(read(command(REG, MONTH)));
	dt.day = bcd2bin(read(command(REG, DAY)));
	dt.year = bcd2bin(read(command(REG, YEAR)));
	dt.dow = read(command(REG, DOW));
	dt.hour = bcd2bin(read(command(REG, HR)) & 0x3F);
	dt.minute = bcd2bin(read(command(REG, MIN)));
	dt.second = bcd2bin(read(command(REG, SEC)) & 0x7F);
	return dt;
}

uint8_t bin2bcd(uint8_t value)
{
	uint8_t temp;
	temp = value/10;
   temp = value - (temp *10) + (temp << 4);
	return temp;
}

uint8_t bcd2bin(uint8_t value)
{
	uint8_t temp;
   //printf("BCD: %x ", value);
	temp = (value & 0x0F) + ((value >> 4) & 0x0F) * 10;
	//printf("BIN: %x\n", temp);
	return temp;
}
